# based on https://developers.google.com/youtube/v3/guides/uploading_a_video

import http.client as httplib
import httplib2
import os
import random
import sys
import time

from apiclient.discovery import build
from apiclient.errors import HttpError
from apiclient.http import MediaFileUpload
from oauth2client.client import flow_from_clientsecrets
from oauth2client.file import Storage
from oauth2client.tools import argparser, run_flow


# Explicitly tell the underlying HTTP transport library not to retry, since
# we are handling retry logic ourselves.
httplib2.RETRIES = 1

# Maximum number of times to retry before giving up.
MAX_RETRIES = 10

# Always retry when these exceptions are raised.
RETRIABLE_EXCEPTIONS = (httplib2.HttpLib2Error, IOError, httplib.NotConnected,
  httplib.IncompleteRead, httplib.ImproperConnectionState,
  httplib.CannotSendRequest, httplib.CannotSendHeader,
  httplib.ResponseNotReady, httplib.BadStatusLine)

# Always retry when an apiclient.errors.HttpError with one of these status
# codes is raised.
RETRIABLE_STATUS_CODES = [500, 502, 503, 504]

# The CLIENT_SECRETS_FILE variable specifies the name of a file that contains
# the OAuth 2.0 information for this application, including its client_id and
# client_secret. You can acquire an OAuth 2.0 client ID and client secret from
# the Google API Console at
# https://console.cloud.google.com/.
# Please ensure that you have enabled the YouTube Data API for your project.
# For more information about using OAuth2 to access the YouTube Data API, see:
#   https://developers.google.com/youtube/v3/guides/authentication
# For more information about the client_secrets.json file format, see:
#   https://developers.google.com/api-client-library/python/guide/aaa_client_secrets
CLIENT_SECRETS_FILE = os.environ.get("CLIENT_SECRETS_FILE", "client_secrets.json")

# This OAuth 2.0 access scope allows an application to upload files to the
# authenticated user's YouTube channel, but doesn't allow other types of access.
YOUTUBE_UPLOAD_SCOPE = "https://www.googleapis.com/auth/youtube.upload"
YOUTUBE_API_SERVICE_NAME = "youtube"
YOUTUBE_API_VERSION = "v3"

# This variable defines a message to display if the CLIENT_SECRETS_FILE is
# missing.
MISSING_CLIENT_SECRETS_MESSAGE = """
WARNING: Please configure OAuth 2.0

To make this sample run you will need to populate the client_secrets.json file
found at:

   %s

with information from the API Console
https://console.cloud.google.com/

For more information about the client_secrets.json file format, please visit:
https://developers.google.com/api-client-library/python/guide/aaa_client_secrets
""" % os.path.abspath(os.path.join(os.path.dirname(__file__),
                                   CLIENT_SECRETS_FILE))

VALID_PRIVACY_STATUSES = ("public", "private", "unlisted")


def get_authenticated_service(client_secrets_file=None):
  flow = flow_from_clientsecrets(client_secrets_file or CLIENT_SECRETS_FILE,
    scope=YOUTUBE_UPLOAD_SCOPE,
    message=MISSING_CLIENT_SECRETS_MESSAGE)

  storage = Storage("%s-oauth2.json" % sys.argv[0])
  credentials = storage.get()

  if credentials is None or credentials.invalid:
    credentials = run_flow(flow, storage)

  return build(YOUTUBE_API_SERVICE_NAME, YOUTUBE_API_VERSION,
    http=credentials.authorize(httplib2.Http()))

def initialize_upload(youtube, title, description, category, tags, privacyStatus, file):
  tags = None

  body=dict(
    snippet=dict(
      title=title,
      description=description,
      tags=tags,
      categoryId=category
    ),
    status=dict(
      privacyStatus=privacyStatus
    )
  )

  # Call the API's videos.insert method to create and upload the video.
  insert_request = youtube.videos().insert(
    part=",".join(body.keys()),
    body=body,
    # The chunksize parameter specifies the size of each chunk of data, in
    # bytes, that will be uploaded at a time. Set a higher value for
    # reliable connections as fewer chunks lead to faster uploads. Set a lower
    # value for better recovery on less reliable connections.
    #
    # Setting "chunksize" equal to -1 in the code below means that the entire
    # file will be uploaded in a single HTTP request. (If the upload fails,
    # it will still be retried where it left off.) This is usually a best
    # practice, but if you're using Python older than 2.6 or if you're
    # running on App Engine, you should set the chunksize to something like
    # 1024 * 1024 (1 megabyte).
    media_body=MediaFileUpload(file, chunksize=-1, resumable=True)
  )

  return resumable_upload(insert_request)

# This method implements an exponential backoff strategy to resume a
# failed upload.
def resumable_upload(insert_request):
  response = None
  error = None
  retry = 0
  while response is None:
    try:
      print("Uploading file...")
      status, response = insert_request.next_chunk()
      if response is not None:
        if 'id' in response:
          print("Video id '%s' was successfully uploaded." % response['id'])
          return response['id']
        else:
          exit("The upload failed with an unexpected response: %s" % response)
          return
    except (HttpError, e):
      if e.resp.status in RETRIABLE_STATUS_CODES:
        error = "A retriable HTTP error %d occurred:\n%s" % (e.resp.status,
                                                             e.content)
      else:
        raise
    except (RETRIABLE_EXCEPTIONS, e):
      error = "A retriable error occurred: %s" % e

    if error is not None:
      print(error)
      retry += 1
      if retry > MAX_RETRIES:
        exit("No longer attempting to retry.")
        return

      max_sleep = 2 ** retry
      sleep_seconds = random.random() * max_sleep
      print("Sleeping %f seconds and then retrying..." % sleep_seconds)
      time.sleep(sleep_seconds)




import datetime
import os
import queue

from apiclient.errors import HttpError
import cv2
from flask import Flask, Response, request, jsonify

from pynopticon import Pynopticon
from pynopticon.upload_video import get_authenticated_service, initialize_upload

q = None
if os.environ.get("CLIENT_SECRETS_FILE"):
  youtube = get_authenticated_service()
else:
  youtube = None

app = Flask(__name__)


def new_frame_handler(frame):
  if q is not None:
    q.put(frame)

cam = int(os.environ.get("CAM", 0))
num_frames = int(os.environ.get("RECORD_FRAMES", 100))

p = Pynopticon(new_frame_callback=new_frame_handler, cam=cam, record_frames=num_frames, youtube=youtube)

def start_receiving():
  global q
  q = queue.Queue()

def stop_receiving():
  global q
  q = None

def generate():
  try:
    start_receiving()
    while True:
      frame = q.get()
      ret, buffer = cv2.imencode(".jpg", frame)
      if not ret:
        continue
      frame = buffer.tobytes()
      yield (b'--frame\r\n'
             b'Content-Type: image/jpeg\r\n\r\n' + frame + b'\r\n')
  except GeneratorExit:
    stop_receiving()


@app.route('/')
def index():
  return Response(generate(), mimetype='multipart/x-mixed-replace; boundary=frame')

@app.route('/stop', methods=["POST"])
def stop():
  p.stop()
  return jsonify({"status": "ok"})

@app.route('/save', methods=["POST"])
def save():
  time = datetime.datetime.now().strftime("%Y-%m-%d_%H-%M-%S")
  fn = time + ".avi"
  upload = request.args.get("upload")
  try:
    upload = (upload == "true")
    vidid = p.save(outname=fn, upload=upload)
    if upload:
      if vidid is None:
        return jsonify({"status": "failed", "message": "Upload failed"})
      return jsonify({"status": "ok", "video_url": f"https://youtu.be/{vidid}"})
  except HttpError as e: 
    return "An HTTP error %d occurred when uploading: %s" % (e.resp.status, e.content)

  return jsonify({"status": "ok", "filename": fn})

@app.route('/start', methods=["POST"])
def start():
  p.start()
  return jsonify({"status": "ok"})

def main():
  host = os.environ.get("HOST", "0.0.0.0")
  port = int(os.environ.get("PORT", 4004))
  p.start()
  app.run(host=host, port=port)

if __name__ == "__main__":
  main()



from distutils.core import setup

with open("README.md", "r") as f:
  long_description = f.read()

setup(name='Pynopticon',
      version='0.0.5',
      description='Pynopticon',
      long_description=long_description,
      long_description_content_type='text/markdown',
      author='Rick Wierenga',
      author_email='rick_wierenga@icloud.com',
      url='https://www.github.com/rickwierenga/pynopticon/',
      packages=['pynopticon'],
      entry_points={
            'console_scripts': [
                  'pynopticon-server = pynopticon.server:main'
            ]   
      } 
   )




import datetime
import queue
import threading

import cv2

from pynopticon.upload_video import initialize_upload, get_authenticated_service


class ClearingQueue(queue.Queue):
  def put(self, item):
    while True:
      try:
        super().put(item, block=False)
      except queue.Full:
        _ = self.get_nowait()
      else:
        break


class Pynopticon:
  def __init__(self, record_frames=100, width=640, height=480, cam=0, new_frame_callback=None, youtube=None):
    self.record_frames = record_frames
    self.queue = ClearingQueue(maxsize=record_frames)
    self.stopped = False
    self.t = None
    self.cap = None

    self.width = width
    self.height = height
    self.cam = cam

    self.new_frame_callback = new_frame_callback
    self.youtube = youtube

  def start(self):
    """ Start the video capture. """
    self.cap = cv2.VideoCapture(self.cam)

    def _record():
      while not self.stopped:
        ret, frame = self.cap.read()
        if not ret:
          break

        frame = cv2.resize(frame, (self.width, self.height))
        self.queue.put(frame)
        if self.new_frame_callback is not None:
          self.new_frame_callback(frame)

    self.stopped = False
    self.t = threading.Thread(target=_record)
    self.t.start()

  def stop(self):
    """ Stop without saving. """
    self.stopped = True
    self.t.join()
    self.cap.release()

  def reset(self):
    """ Reset queue. """
    self.queue = ClearingQueue(maxsize=self.record_frames)

  def save(self, outname: str = "output.avi", fps: int = 15, upload=False, title=None, description=None):
    """ Stop and save. """
    self.stop()

    out = cv2.VideoWriter(outname, cv2.VideoWriter_fourcc(*"DIVX"), fps, (self.width, self.height))

    while True:
      try:
        frame = self.queue.get_nowait()
        out.write(frame)
      except queue.Empty:
        break

    out.release()

    if upload:
      assert self.youtube is not None, "Youtube client not initialized"

      time = datetime.datetime.now().strftime("%Y-%m-%d_%H-%M-%S")

      try:
        vidid = initialize_upload(self.youtube,
          title=title or f"Epic fail: {outname}",
          description=description or ("Pynopticon capture from " + time),
          category="22",
          tags=[],
          privacyStatus="private",
          file=outname)
        return vidid
      except HttpError as e: 
        return "An HTTP error %d occurred when uploading: %s" % (e.resp.status, e.content)
    
    return None
